
#if !defined(AFX_ABSTRACTSYNTAXCOLORER_INCLUDED_)
#define AFX_ABSTRACTSYNTAXCOLORER_INCLUDED_

class AbstractSyntaxColorer
{
	friend class CCrystalTextView;
public:
	AbstractSyntaxColorer();

	COLORREF defaultTextColor;
	COLORREF defaultBkColor;
	COLORREF noTextColor;
	COLORREF selectTextColor;
	COLORREF selectBkColor;

	virtual void parseLine(const char *line, DWORD &state, int &lineFlags) = 0;

	void color(int begin, int end, COLORREF crText, COLORREF crBK);
	void startColor(int begin, COLORREF crText, COLORREF crBK);
	void endColor(int end);

	virtual DWORD createParseState() { return 0; }
	virtual DWORD copyParseState(DWORD state) { return state; }
	virtual void destroyParseState(DWORD &state) { state = 0; }

private:
	void drawLine(LPCTSTR line, DWORD &state, int &lineFlags);
	CCrystalTextView *_view;
	bool _activeColorRange;
	int _begin;
	COLORREF _crText;
	COLORREF _crBK;
};

template <class S>
class AbstractStateSyntaxColorer : public AbstractSyntaxColorer
{
	virtual void parseLine(const char *line, S& state, int &lineFlags) = 0;
	void parseLine(const char *line, DWORD &state, int &lineFlags)
	{	parseLine(line, *(S*)state, lineFlags);
	}
	DWORD createParseState() { return (DWORD)new S; }
	DWORD copyParseState(DWORD state) { return (DWORD)new S((S*)state); }
	void destroyParseState(DWORD &state) 
	{
		ASSERT(state != 0);
		if (state != 0)
			delete (S*)state;
		state = 0;
	}
};

#endif // AFX_ABSTRACTSYNTAXCOLORER_INCLUDED_