// SampleImageView.cpp : implementation of the CSampleImageView class
//

#include "stdafx.h"
#include "Sample.h"

#include "SampleImageDoc.h"
#include "SampleImageView.h"
#include "editcmd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSampleImageView

IMPLEMENT_DYNCREATE(CSampleImageView, CView)

BEGIN_MESSAGE_MAP(CSampleImageView, CView)
	//{{AFX_MSG_MAP(CSampleImageView)
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	ON_WM_VSCROLL()
	ON_WM_HSCROLL()
	ON_WM_CONTEXTMENU()
	ON_WM_CHAR()
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
	// Syntax colorer
	ON_COMMAND(ID_VIEW_COLORER_CPP, OnSelectCppColorer)
	ON_COMMAND(ID_VIEW_COLORER_CDF, OnSelectCDFColorer)
	ON_COMMAND(ID_VIEW_COLORER_HTML, OnSelectHTMLColorer)
	ON_COMMAND(ID_VIEW_COLORER_XML, OnSelectXMLColorer)
	ON_COMMAND(ID_VIEW_COLORER_JAVASCRIPT, OnSelectJavaScriptColorer)
	ON_COMMAND(ID_VIEW_COLORER_CVB, OnSelectcvbColorer)
	ON_COMMAND(ID_VIEW_COLORER_NONE, OnSelectNoColorer)
	//	Status
	//ON_UPDATE_COMMAND_UI(ID_EDIT_INDICATOR_CRLF, OnUpdateIndicatorCRLF)
	ON_UPDATE_COMMAND_UI(ID_EDIT_INDICATOR_POSITION, OnUpdateIndicatorPosition)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSampleImageView construction/destruction

CSampleImageView::CSampleImageView()
{
	AFX_ZERO_INIT_OBJECT(CView);
	m_nMagnification = 1;
}

CSampleImageView::~CSampleImageView()
{
	ASSERT(m_pCacheBitmap == NULL);
}

BOOL CSampleImageView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}


/////////////////////////////////////////////////////////////////////////////
// CSampleImageView diagnostics

#ifdef _DEBUG
void CSampleImageView::AssertValid() const
{
	CView::AssertValid();
}

void CSampleImageView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CSampleImageDoc* CSampleImageView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSampleImageDoc)));
	return (CSampleImageDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSampleImageView message handlers

void CSampleImageView::OnDestroy() 
{
	CView::OnDestroy();

	if (m_pCacheBitmap != NULL)
	{
		m_pCacheBitmap->DeleteObject();
		delete m_pCacheBitmap;
		m_pCacheBitmap = NULL;
	}
}

BOOL CSampleImageView::OnEraseBkgnd(CDC *pdc) 
{
	return TRUE;
}

void CSampleImageView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();

	//	Update scrollbars
	RecalcVertScrollBar();
	RecalcHorzScrollBar();
}

void CSampleImageView::OnSetFileName(LPCTSTR lpszPathName)
{
	const char *ext;
	for (ext = lpszPathName + strlen(lpszPathName); ext > lpszPathName && ext[-1] != '.'; ext--);
}

void CSampleImageView::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	//AfxMessageBox("Build your own context menu!");
}

void CSampleImageView::OnDraw(CDC *pDC)
{
	if ( GetDocument() == 0 || pDC == 0){
		ASSERT( 0 );
		return;
	}

	CRect rcClient;
	GetClientRect( &rcClient );
	CRect clippingRect (0, 0, 0, 0);
	if (pDC->GetClipBox(&clippingRect) == NULLREGION)
		clippingRect = rcClient;

	//clippingRect.InflateRect( 1, 1 );
 
	CDC cacheDC;
	VERIFY(cacheDC.CreateCompatibleDC(pDC));
	if (m_pCacheBitmap == NULL)
	{
		m_pCacheBitmap = new CBitmap;
		VERIFY(m_pCacheBitmap->CreateCompatibleBitmap(pDC, rcClient.Width(), rcClient.Height()));
	}
	CBitmap *pOldBitmap = cacheDC.SelectObject(m_pCacheBitmap);

	cacheDC.FillSolidRect(rcClient, RGB(128, 128, 128)); //pDC->GetBkColor());

	m_nXoffset = 0;
	int nImageWidth = GetImageWidth();
	int nExtraWidth = rcClient.Width() - nImageWidth;
	if (m_nLeftX == 0 && nExtraWidth > 0)
		m_nXoffset = nExtraWidth/2;

	m_nYoffset = 0;
	int nImageHeight = GetImageHeight();
	int nExtraHeight = rcClient.Height() - nImageHeight;
	if (m_nTopY == 0 && nExtraHeight > 0)
		m_nYoffset = nExtraHeight/2;

	GetDocument()->m_gflBitmap.drawImageStretch( cacheDC,
		m_nXoffset, m_nYoffset, nImageWidth, nImageHeight, 
		m_nLeftX / m_nMagnification, -m_nTopY / m_nMagnification ); 
		
	
	VERIFY(pDC->BitBlt(rcClient.left, rcClient.top, rcClient.Width(), rcClient.Height(),
					   &cacheDC, rcClient.left, rcClient.top, SRCCOPY));
	cacheDC.SelectObject(pOldBitmap);
	cacheDC.DeleteDC();

}


void CSampleImageView::RecalcVertScrollBar(BOOL bPositionOnly /*= FALSE*/)
{
	SCROLLINFO si;
	si.cbSize = sizeof(si);
	if (bPositionOnly)
	{
		si.fMask = SIF_POS;
		si.nPos = m_nTopY;
	}
	else
	{
		if (GetScreenHeight() >= GetImageHeight() && m_nTopY > 0)
		{
			m_nTopY = 0;
			Invalidate();
		}
		si.fMask = SIF_DISABLENOSCROLL | SIF_PAGE | SIF_POS | SIF_RANGE;
		si.nMin = 0;
		si.nMax = GetImageHeight() - 1;
		si.nPage = GetScreenHeight();
		si.nPos = m_nTopY;
	}
	VERIFY(SetScrollInfo(SB_VERT, &si));
}

void CSampleImageView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	CView::OnVScroll(nSBCode, nPos, pScrollBar);

	//	Note we cannot use nPos because of its 16-bit nature
	SCROLLINFO si;
	si.cbSize = sizeof(si);
	si.fMask = SIF_ALL;
	VERIFY(GetScrollInfo(SB_VERT, &si));

	int nImageHeight = GetImageHeight();

	int nNewTopY;
	BOOL bDisableSmooth = TRUE;
	switch (nSBCode)
	{
	case SB_TOP:
		nNewTopY = 0;
		bDisableSmooth = FALSE;
		break;
	case SB_BOTTOM:
		nNewTopY = m_nTopY - GetScreenHeight() + 1;
		bDisableSmooth = FALSE;
		break;
	case SB_LINEUP:
		nNewTopY = m_nTopY - 8;
		break;
	case SB_LINEDOWN:
		nNewTopY = m_nTopY + 8;
		break;
	case SB_PAGEUP:
		nNewTopY = m_nTopY - si.nPage + 1;
		bDisableSmooth = FALSE;
		break;
	case SB_PAGEDOWN:
		nNewTopY = m_nTopY + si.nPage - 1;
		bDisableSmooth = FALSE;
		break;
	case SB_THUMBPOSITION:
	case SB_THUMBTRACK:
		nNewTopY = si.nTrackPos;
		break;
	default:
		return;
	}

	if (nNewTopY < 0)
		nNewTopY = 0;
	if (nNewTopY >= nImageHeight)
		nNewTopY = nImageHeight - 1;

	if (m_nTopY != nNewTopY)
	{
		//int nScrollY = m_nTopY - nNewTopY;
		m_nTopY = nNewTopY;
		Invalidate();
		//ScrollWindow(0, nScrollY);
		//UpdateWindow();
		RecalcVertScrollBar(TRUE);
	}
}

void CSampleImageView::RecalcHorzScrollBar(BOOL bPositionOnly /*= FALSE*/)
{
	//	Again, we cannot use nPos because it's 16-bit
	SCROLLINFO si;
	si.cbSize = sizeof(si);
	if (bPositionOnly)
	{
		si.fMask = SIF_POS;
		si.nPos = m_nLeftX;
	}
	else
	{
		if (GetScreenWidth() >= GetImageWidth() && m_nLeftX > 0)
		{
			m_nLeftX = 0;
			Invalidate();
			//UpdateCaret();
		}
		si.fMask = SIF_DISABLENOSCROLL | SIF_PAGE | SIF_POS | SIF_RANGE;
		si.nMin = 0;
		si.nMax = GetImageWidth() - 1;
		si.nPage = GetScreenWidth();
		si.nPos = m_nLeftX;
	}
	VERIFY(SetScrollInfo(SB_HORZ, &si));
}

void CSampleImageView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	CView::OnHScroll(nSBCode, nPos, pScrollBar);

	SCROLLINFO si;
	si.cbSize = sizeof(si);
	si.fMask = SIF_ALL;
	VERIFY(GetScrollInfo(SB_HORZ, &si));

	int nImageWidth = GetImageWidth();

	int nNewLeftX;
	switch (nSBCode)
	{
	case SB_LEFT:
		nNewLeftX = 0;
		break;
	case SB_RIGHT:
		nNewLeftX = nImageWidth - GetScreenWidth() + 1;
		break;
	case SB_LINEUP:
		nNewLeftX = m_nLeftX - 1;
		break;
	case SB_LINEDOWN:
		nNewLeftX = m_nLeftX + 1;
		break;
	case SB_PAGEUP:
		nNewLeftX = m_nLeftX - si.nPage + 1;
		break;
	case SB_PAGEDOWN:
		nNewLeftX = m_nLeftX + si.nPage - 1;
		break;
	case SB_THUMBPOSITION:
	case SB_THUMBTRACK:
		nNewLeftX = si.nTrackPos;
		break;
	default:
		return;
	}

	if (nNewLeftX < 0)
		nNewLeftX = 0;
	if (nNewLeftX >= nImageWidth)
		nNewLeftX = nImageWidth - 1;

	if (m_nLeftX != nNewLeftX)
	{
		int nScrollX = m_nLeftX - nNewLeftX;
		m_nLeftX = nNewLeftX;
		//CRect rcScroll;
		//GetClientRect(&rcScroll);
		//ScrollWindow(nScrollX, 0, &rcScroll, &rcScroll);
		//UpdateWindow();
		Invalidate();
		RecalcHorzScrollBar(TRUE);
	}
}

void CSampleImageView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);

	if (m_pCacheBitmap != NULL)
	{
		m_pCacheBitmap->DeleteObject();
		delete m_pCacheBitmap;
		m_pCacheBitmap = NULL;
	}
	m_nScreenWidth = -1;
	m_nScreenHeight = -1;
	RecalcVertScrollBar();
	RecalcHorzScrollBar();
}

void CSampleImageView::OnUpdateIndicatorPosition(CCmdUI* pCmdUI)
{
	int x = (m_ptMousePos.x - m_nXoffset - 1)/m_nMagnification;
	int y = (m_ptMousePos.y - m_nYoffset - 1)/m_nMagnification;

	CString stat;
	GFLBitmap& bitmap = GetDocument()->m_gflBitmap;
	if (   0 <= x && x < bitmap.width()
		&& 0 <= y && y < bitmap.height())
		stat.Format(_T("%d, %d"), x+1, y+1);
	pCmdUI->SetText(stat);
}

void CSampleImageView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CView::OnMouseMove(nFlags, point);

	m_ptMousePos = point;
}

int CSampleImageView::GetScreenWidth()
{
	if (m_nScreenWidth == -1)
	{
		CRect rect;
		GetClientRect(&rect);
		m_nScreenWidth = rect.Width();
	}
	return m_nScreenWidth;
}

int CSampleImageView::GetScreenHeight()
{
	if (m_nScreenHeight == -1)
	{
		CRect rect;
		GetClientRect(&rect);
		m_nScreenHeight = rect.Height();
	}
	return m_nScreenHeight;
}

int CSampleImageView::GetImageWidth()
{
	return GetDocument()->m_gflBitmap.width() * m_nMagnification;
}

int CSampleImageView::GetImageHeight()
{
	return GetDocument()->m_gflBitmap.height() * m_nMagnification;
}

void CSampleImageView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	CView::OnChar(nChar, nRepCnt, nFlags);

	if (nChar == '+')
	{
		if (m_nMagnification < 16)
		{
			m_nMagnification *= 2;
			RecalcVertScrollBar();
			RecalcHorzScrollBar();
			Invalidate();
		}
	}
	else if (nChar == '-')
	{
		if (m_nMagnification > 1)
		{
			m_nMagnification /= 2;
			RecalcVertScrollBar();
			RecalcHorzScrollBar();
			Invalidate();
		}
	}
}

void CSampleImageView::OnSelectCppColorer()
{
}

void CSampleImageView::OnSelectCDFColorer()
{
}

void CSampleImageView::OnSelectHTMLColorer()
{
}

void CSampleImageView::OnSelectXMLColorer()
{
}

void CSampleImageView::OnSelectJavaScriptColorer()
{
}

void CSampleImageView::OnSelectcvbColorer()
{
}

void CSampleImageView::OnSelectNoColorer()
{
}

