// SampleView.cpp : implementation of the CSampleView class
//

#include "stdafx.h"
#include "Sample.h"

#include "SampleDoc.h"
#include "SampleView.h"
#include "CppSyntaxColorer.h"
#include "CDFSyntaxColorer.h"
#include "HTMLSyntaxColorer.h"
#include "XMLSyntaxColorer.h"
#include "JavaScriptSyntaxColorer.h"
#include "cvbSyntaxColorer.h"
#include "editcmd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSampleView

IMPLEMENT_DYNCREATE(CSampleView, CCrystalEditView)

BEGIN_MESSAGE_MAP(CSampleView, CCrystalEditView)
	//{{AFX_MSG_MAP(CSampleView)
	ON_WM_CONTEXTMENU()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CCrystalEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CCrystalEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CCrystalEditView::OnFilePrintPreview)
	// Syntax colorer
	ON_COMMAND(ID_VIEW_COLORER_CPP, OnSelectCppColorer)
	ON_COMMAND(ID_VIEW_COLORER_CDF, OnSelectCDFColorer)
	ON_COMMAND(ID_VIEW_COLORER_HTML, OnSelectHTMLColorer)
	ON_COMMAND(ID_VIEW_COLORER_XML, OnSelectXMLColorer)
	ON_COMMAND(ID_VIEW_COLORER_JAVASCRIPT, OnSelectJavaScriptColorer)
	ON_COMMAND(ID_VIEW_COLORER_CVB, OnSelectcvbColorer)
	ON_COMMAND(ID_VIEW_COLORER_NONE, OnSelectNoColorer)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSampleView construction/destruction

CSampleView::CSampleView()
{
	// TODO: add construction code here

}

CSampleView::~CSampleView()
{
}

BOOL CSampleView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CCrystalEditView::PreCreateWindow(cs);
}


/////////////////////////////////////////////////////////////////////////////
// CSampleView diagnostics

#ifdef _DEBUG
void CSampleView::AssertValid() const
{
	CCrystalEditView::AssertValid();
}

void CSampleView::Dump(CDumpContext& dc) const
{
	CCrystalEditView::Dump(dc);
}

CSampleDoc* CSampleView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSampleDoc)));
	return (CSampleDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSampleView message handlers

CCrystalTextBuffer *CSampleView::LocateTextBuffer()
{
	return &GetDocument()->m_xTextBuffer;
}

void CSampleView::OnInitialUpdate() 
{
	CCrystalEditView::OnInitialUpdate();

	SetFont(GetDocument()->m_lf);

	CCrystalTextBuffer *textbuf = LocateTextBuffer();
	if (textbuf != 0 && !textbuf->sFileName.IsEmpty())
		OnSetFileName(textbuf->sFileName);
}

void CSampleView::OnSetFileName(LPCTSTR lpszPathName)
{
	const char *ext;
	for (ext = lpszPathName + strlen(lpszPathName); ext > lpszPathName && ext[-1] != '.'; ext--);

	if (   stricmp(ext, "cpp") == 0 
		|| stricmp(ext, "c") == 0 
		|| stricmp(ext, "h") == 0)
		SetSyntaxColorer(new CppSyntaxColorer);
	else if (stricmp(ext, "cdf") == 0)
		SetSyntaxColorer(new CDFSyntaxColorer);
	else if (stricmp(ext, "html") == 0 || stricmp(ext, "htm") == 0)
		SetSyntaxColorer(new HTMLSyntaxColorer);
	else if (stricmp(ext, "xml") == 0)
		SetSyntaxColorer(new XMLSyntaxColorer);
	else if (stricmp(ext, "js") == 0)
		SetSyntaxColorer(new JavaScriptSyntaxColorer);
	else if (stricmp(ext, "SBR") == 0)
		SetSyntaxColorer(new cvbSyntaxColorer);
}

void CSampleView::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	//AfxMessageBox("Build your own context menu!");
}

void CSampleView::OnSelectCppColorer()
{
	SetSyntaxColorer(new CppSyntaxColorer);
}

void CSampleView::OnSelectCDFColorer()
{
	SetSyntaxColorer(new CDFSyntaxColorer);
}

void CSampleView::OnSelectHTMLColorer()
{
	SetSyntaxColorer(new HTMLSyntaxColorer);
}

void CSampleView::OnSelectXMLColorer()
{
	SetSyntaxColorer(new XMLSyntaxColorer);
}

void CSampleView::OnSelectJavaScriptColorer()
{
	SetSyntaxColorer(new JavaScriptSyntaxColorer);
}

void CSampleView::OnSelectcvbColorer()
{
	SetSyntaxColorer(new cvbSyntaxColorer);
}

void CSampleView::OnSelectNoColorer()
{
	SetSyntaxColorer(0);
}

