#include "stdafx.h"
#include "XMLSyntaxColorer.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

XMLSyntaxColorState::XMLSyntaxColorState()
{
	state = 0;
}

XMLSyntaxColorState::XMLSyntaxColorState(XMLSyntaxColorState* original)
{
	stateStack = original->stateStack;
	state = original->state;
}

#define ST_TEXT				0
#define ST_IN_TAG_OPEN		1	// between < and tagname
#define ST_IN_TAG			2	// after tagname or attribute
#define ST_IN_TAG_ATTR		3	// after attribute name
#define ST_IN_TAG_IS		4	// after =
#define ST_IN_TAG_CLOSE		6	// after /
#define ST_IN_TAG_META		7	// meta tag starting with !
#define ST_IN_TAG_COMM		8	// inside comment
#define ST_IN_TAG_COMM2		9	// inside comment (meta tag) -- the usual case


#define COLOR_WHITESPACE	RGB(255, 255, 255)
#define COLOR_NORMALTEXT	RGB(  0,   0,   0)
//#define COLOR_PREPROCESSOR	RGB(128,   0,   0)
#define COLOR_COMMENT		RGB( 64,  64, 220)
//#define COLOR_NUMBER		RGB(128,   0,   0)
//#define COLOR_OPERATOR		RGB(  0,   0,   0)
//#define COLOR_STRING		RGB(  0, 128,   0)
//#define COLOR_BK_STRING		RGB(240, 255, 240)
//#define COLOR_KEYWORD		RGB(  0,  64, 128)
//#define COLOR_SELBKGND		RGB(  0,   0,   0)
//#define COLOR_SELTEXT		RGB(255, 255, 255)
#define COLOR_TEXT_ERROR	RGB(255,   0,   0)
#define COLOR_BK_ERROR		RGB(255, 240, 240)
#define COLOR_TEXT_TAG		RGB( 64,   0,  64)
#define COLOR_BK_TAG		RGB(255, 250, 255)
#define COLOR_TEXT_VAL		RGB(  0, 128,   0)
#define COLOR_BK_VAL		RGB(245, 255, 245)

XMLSyntaxColorer::XMLSyntaxColorer()
{
	defaultTextColor = ::GetSysColor(COLOR_WINDOWTEXT);
	defaultBkColor = ::GetSysColor(COLOR_WINDOW);
	noTextColor = RGB(250, 250, 250);
	selectTextColor = RGB(255, 255, 255);
	selectBkColor = RGB(0, 0, 0);
}

void XMLSyntaxColorer::parseLine(const char* line, XMLSyntaxColorState &XMLstate, int &lineFlags)
{
	SharedStack<CString> *stateStack = &XMLstate.stateStack;
	int state = XMLstate.state;

	int i = 0;
	while (line[i] != '\0')
	{
		switch(state)
		{
		case ST_TEXT:
			if (line[i] == '<')
			{	startColor(i, COLOR_TEXT_TAG, COLOR_BK_TAG);
				state = ST_IN_TAG_OPEN;
				i++;
			}
			else if (line[i] == '>')
			{	startColor(i, COLOR_TEXT_ERROR, COLOR_BK_ERROR);
				i++;
				endColor(i);
			}
			else if (line[i] == '&')
			{	int begin = i;
				bool error = false;
				i++;
				if (line[i] == '#')
				{
					i++;
					if (line[i] == 'x')
					{	i++;
						for (; isdigit(line[i]) || ('a' <= line[i] && line[i] <= 'f'); i++);
					}
					else
						for (; isdigit(line[i]); i++);
				}
				else
				{
					char chcode[30];
					int j = 0;
					while (   line[i] != ';' && line[i] != '\0' 
						   && line[i] != '<' && line[i] != '>' && line[i] != '&')
					{	if (j < 29)
							chcode[j++] = line[i];
						i++;
					}
					chcode[j] = 0;
				}
				if (line[i] == ';')
					i++;
				else
					error = true;
				if (error)
					color(begin, i, COLOR_TEXT_ERROR, COLOR_BK_ERROR);
			}
			else
				i++;
			break;
		case ST_IN_TAG_OPEN:
			startColor(i, COLOR_TEXT_TAG, COLOR_BK_TAG);
			if (line[i] == ' ' || line[i] == '\t')
				i++;
			else if (line[i] == '!')
			{	state = ST_IN_TAG_META;
				i++;
			}
			else if (isalpha(line[i]) || line[i] == '_')
			{	int begin = i;
				char tagname[100];
				int j = 0;
				while (isalnum(line[i]) || line[i] == '_')
				{	if (j < 99)
						tagname[j++] = tolower(line[i]);
					i++;
				}
				tagname[j] = '\0';
				const CString str(tagname);
				stateStack->push(str);
				state = ST_IN_TAG;
			}
			else if (line[i] == '/')
			{	int begin = i;
				i++;
				char tagname[100];
				int j = 0;
				while (isalnum(line[i]) || line[i] == '_')
				{	if (j < 99)
						tagname[j++] = tolower(line[i]);
					i++;
				}
				tagname[j] = '\0';
				const CString str(tagname);
				if (!stateStack->isEmpty() && stateStack->top() == str)
					stateStack->pop();
				else
					startColor(begin, COLOR_TEXT_ERROR, COLOR_BK_ERROR);
				state = ST_IN_TAG_CLOSE;
			}
			else
			{	startColor(i, COLOR_TEXT_ERROR, COLOR_BK_ERROR);
				i++;
			}
			break;
		case ST_IN_TAG:
			startColor(i, COLOR_TEXT_TAG, COLOR_BK_TAG);
			if (line[i] == ' ' || line[i] == '\t')
				i++;
			else if (line[i] == '-' && line[i+1] == '-')
			{	state = ST_IN_TAG_COMM;
				startColor(i, COLOR_COMMENT, defaultBkColor);
				i += 2;
			}
			else if (isalpha(line[i]) || line[i] == '_' || line[i] == '.')
			{	int begin = i;
				while (isalnum(line[i]) || line[i] == '_' || line[i] == '.') i++;
				state = ST_IN_TAG_ATTR;
			}
			else if (line[i] == '>')
			{	startColor(i, COLOR_TEXT_TAG, COLOR_BK_TAG);
				state = ST_TEXT;
				i++;
				endColor(i);
			}
			else if (line[i] == '/')
			{	i++;
				if (!stateStack->isEmpty())
					stateStack->pop();
				state = ST_IN_TAG_CLOSE;
			}
			else
			{	startColor(i, COLOR_TEXT_ERROR, COLOR_BK_ERROR);
				i++;
			}
			break;
		case ST_IN_TAG_ATTR:
			startColor(i, COLOR_TEXT_TAG, COLOR_BK_TAG);
			if (line[i] == ' ' || line[i] == '\t')
				i++;
			else if (isalpha(line[i]) || line[i] == '_')
			{	state = ST_IN_TAG;
			}
			else if (line[i] == '=')
			{	state = ST_IN_TAG_IS;
				i++;
			}
			else if (line[i] == '>')
			{	startColor(i, COLOR_TEXT_TAG, COLOR_BK_TAG);
				state = ST_TEXT;
				i++;
				endColor(i);
			}
			else if (line[i] == '/')
			{	i++;
				if (!stateStack->isEmpty())
					stateStack->pop();
				state = ST_IN_TAG_CLOSE;
			}
			else
			{	startColor(i, COLOR_TEXT_ERROR, COLOR_BK_ERROR);
				i++;
			}
			break;
		case ST_IN_TAG_IS:
			startColor(i, COLOR_TEXT_TAG, COLOR_BK_TAG);
			if (line[i] == ' ' || line[i] == '\t')
				i++;
			else if (line[i] == '"')
			{
				int begin = i;
				i++;
				while (line[i] != '\0' && line[i] != '"')
					i++;
				if (line[i] == '"')
				{	startColor(begin, COLOR_TEXT_VAL, COLOR_BK_TAG);
					i++;
					startColor(i, COLOR_TEXT_TAG, COLOR_BK_TAG);
				}
				else
					startColor(begin, COLOR_TEXT_ERROR, COLOR_BK_ERROR);
				state = ST_IN_TAG;
			}
			else if (line[i] == '>')
			{	startColor(i, COLOR_TEXT_ERROR, COLOR_BK_ERROR);
				i++;
				endColor(i);
			}
			else
			{	
				startColor(i, COLOR_TEXT_VAL, COLOR_BK_TAG);
				while (line[i] != '\0' && line[i] != ' ' && line[i] != '>')
					i++;
				startColor(i, COLOR_TEXT_TAG, COLOR_BK_TAG);
				state = ST_IN_TAG;
			}
			break;
		case ST_IN_TAG_COMM:
		case ST_IN_TAG_COMM2:
			startColor(i, COLOR_COMMENT, defaultBkColor);
			while (line[i] != '\0')
			{	if (line[i] == '-' && line[i+1] == '-')
				{	state = state == ST_IN_TAG_COMM ? ST_IN_TAG : ST_IN_TAG_META;
					i += 2;
					endColor(i);
					break;
				}
				i++;
			}
			break;
		case ST_IN_TAG_CLOSE:
			if (line[i] == ' ' || line[i] == '\t')
				i++;
			else if (line[i] == '>')
			{	startColor(i, COLOR_TEXT_TAG, COLOR_BK_TAG);
				state = ST_TEXT;
				i++;
				endColor(i);
			}
			else
			{	startColor(i, COLOR_TEXT_ERROR, COLOR_BK_ERROR);
				i++;
				state = ST_IN_TAG;
			}
			break;
		case ST_IN_TAG_META:
			if (line[i] == ' ' || line[i] == '\t')
				i++;
			else if (line[i] == '-' && line[i+1] == '-')
			{	state = ST_IN_TAG_COMM2;
				startColor(i, COLOR_COMMENT, defaultBkColor);
				i += 2;
			}
			else if (isalpha(line[i]) || line[i] == '_')
			{	while (line[i] != '\0' && line[i] != ' ' && line[i] != '\t'
					   && line[i] != '>')
					i++;
			}
			else if (line[i] == '"')
			{
				int begin = i;
				i++;
				while (line[i] != '\0' && line[i] != '"')
					i++;
				if (line[i] == '"')
				{	startColor(begin, COLOR_TEXT_VAL, COLOR_BK_TAG);
					i++;
					startColor(i, COLOR_TEXT_TAG, COLOR_BK_TAG);
				}
			}
			else if (line[i] == '>')
			{	startColor(i, COLOR_TEXT_TAG, COLOR_BK_TAG);
				state = ST_TEXT;
				i++;
				endColor(i);
			}
			
			else
			{	startColor(i, COLOR_TEXT_ERROR, COLOR_BK_ERROR);
				i++;
			}
		}
	}

	XMLstate.state = state;
}
